package com.gic.haoban.manage.service.service.out.impl.licence;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderPageDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderProgressDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderQDTO;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.licence.LicenceOrderApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrder;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrderProgress;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.licence.LicenceOrderProgressService;
import com.gic.haoban.manage.service.service.licence.LicenceOrderService;
import com.gic.haoban.manage.service.util.DingUtils;
import com.gic.wechat.api.dto.qywx.fee.CreateOrderResponseDTO;
import com.gic.wechat.api.dto.qywx.fee.qdto.CreateOrderQDTO;
import com.gic.wechat.api.service.qywx.QywxOrderApiService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @description:
 * @Author: wenhua
 * @Date: 2023/3/9 17:04
 */
@Service("licenceOrderApiServiceImpl")
public class LicenceOrderApiServiceImpl implements LicenceOrderApiService {

    private static final Logger logger = LogManager.getLogger(LicenceOrderApiServiceImpl.class);

    @Autowired
    private LicenceOrderService licenceOrderService;
    @Autowired
    private LicenceOrderProgressService licenceOrderProgressService;

    @Autowired
    private EnterpriseService enterpriseService;

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;

    @Autowired
    private QywxOrderApiService qywxOrderApiService;
    @Autowired
    private Config config;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Override
    public ServiceResponse<LicenceOrderDTO> getLicenceOrderDetail(Long orderId,Integer type) {
        TabHaobanLicenceOrder licenceOrderDetail = licenceOrderService.getLicenceOrderDetail(orderId);
        if (licenceOrderDetail != null){
            //查询订单进度表
            LicenceOrderDTO licenceOrderDTO = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderDetail), LicenceOrderDTO.class);
            List<TabHaobanLicenceOrderProgress> list = licenceOrderProgressService.getListByOrderId(orderId,type);
            licenceOrderDTO.setOrderProgressList(EntityUtil.changeEntityListByJSON(LicenceOrderProgressDTO.class, list));
            //查询gic品牌名称
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(licenceOrderDTO.getEnterpriseId());
            if (ObjectUtil.isNotNull(enterpriseDTO)){
                licenceOrderDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
            }
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(licenceOrderDTO.getWxEnterpriseId());
            if (ObjectUtil.isNotNull(wxEnterpriseDTO)){
                logger.info("企业微信查询返回实体：{}", JSON.toJSONString(wxEnterpriseDTO));
                licenceOrderDTO.setWxEnterpriseName(wxEnterpriseDTO.getCorpName());
            }
            return ServiceResponse.success(licenceOrderDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> deleteLicenceOrder(Long orderId) {
        licenceOrderService.deleteLicenceOrder(orderId);
        licenceOrderProgressService.deleteByOrderId(orderId);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<String> saveOrUpdateLicenceOrder(LicenceOrderQDTO licenceOrderQDTO) {
        String s = licenceOrderService.saveOrUpdateLicenceOrder(licenceOrderQDTO);
        return ServiceResponse.success(s);
    }

    @Override
    public ServiceResponse<Page<LicenceOrderPageDTO>> getLicenceOrderPage(LicenceOrderPageQDTO licenceOrderPageQDTO) {
        Page<LicenceOrderPageDTO> licenceOrderPage = licenceOrderService.getLicenceOrderPage(licenceOrderPageQDTO);
        return ServiceResponse.success(licenceOrderPage);
    }

    @Override
    public ServiceResponse<Boolean> updateLicenceOrderType(Long orderId, Integer type,String enterpriseId,String wxEnterpriseId,String creatorName,Integer wxFlag) {
        licenceOrderService.updateLicenceOrderType(orderId,type,enterpriseId,wxEnterpriseId,creatorName,wxFlag);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> uploadLicenceOrderVoucher(Long orderId, String voucher,String enterpriseId,String wxEnterpriseId,String creatorName) {
        licenceOrderService.uploadLicenceOrderVoucher(orderId,voucher,enterpriseId,wxEnterpriseId,creatorName);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> payLicenceOrder(Long orderId,String wxEnterpriseId) {
        //企业微信下单
        TabHaobanLicenceOrder licenceOrderDetail = licenceOrderService.getLicenceOrderDetail(orderId);
        if (licenceOrderDetail == null){
            return ServiceResponse.success(false);
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        String openCorpid = wxEnterpriseDTO.getOpenCorpid();
        String serviceCorpid = config.getCorpid();
        CreateOrderQDTO orderQDTO = new CreateOrderQDTO();
        orderQDTO.setBaseCount(0);
        orderQDTO.setExternalCount(licenceOrderDetail.getExternalContactCount());
        //todo 购买人userId，加密 先写死
        orderQDTO.setBuyerUserid("wo59NLDQAA_8N2VFOP3qPubiCqxi0LGg");
        if (licenceOrderDetail.getTimeType() == 1){
            orderQDTO.setMonths(licenceOrderDetail.getTimeValue());
            orderQDTO.setDays(0);
        }else {
            orderQDTO.setMonths(0);
            orderQDTO.setDays(licenceOrderDetail.getTimeValue());
        }
        ServiceResponse<CreateOrderResponseDTO> newOrder = qywxOrderApiService.createNewOrder(serviceCorpid,openCorpid,orderQDTO);
        logger.info("企业微信订单返回信息：{}",JSON.toJSONString(newOrder));
        if (newOrder.isSuccess()){
            //更新企业微信订单状态，保存企业微信订单id
            CreateOrderResponseDTO result = newOrder.getResult();
            licenceOrderService.updateLicenceOrderType(licenceOrderDetail.getOrderId(),0,licenceOrderDetail.getEnterpriseId(),
                    licenceOrderDetail.getWxEnterpriseId(),"系统",0);

            licenceOrderService.saveQywxOrderId(licenceOrderDetail.getOrderId(),result.getOrderId());
            return ServiceResponse.success(true);
        }
        return ServiceResponse.success(false);
    }

    @Override
    public ServiceResponse<String> isPayLicenceOrder(String wxEnterpriseId) {
        String order = licenceOrderService.isPayLicenceOrder(wxEnterpriseId);
        return ServiceResponse.success(order);
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> wxCallBack(String params) {
        logger.info("接收微信回调参数：{}",params);
        if(StrUtil.isBlank(params)){
            return ServiceResponse.success(false);
        }
        JSONObject object = JSONObject.parseObject(params, JSONObject.class);
        //微信的订单id，保留方便后续对账
        String orderId = object.getString("orderId");
        String transactionId = object.getString("transactionId");
        String timeEnd = object.getString("timeEnd");
        TabHaobanLicenceOrder licenceOrder = licenceOrderService.selectByTransactionId(orderId);
        if (licenceOrder == null){
            logger.info("微信回调的订单编号：{}异常！！",orderId);
            return ServiceResponse.success(false);
        }
        if (licenceOrder.getOrderStatus() == 1){
            //微信会多次回调
            logger.info("微信回调的订单已处理！");
            return ServiceResponse.success(false);
        }
        Date payTime = DateUtil.strToDate(DateUtil.FORMAT_DATETIME_14, timeEnd);
        //更新订单状态
        licenceOrderService.updateLicenceOrderType(licenceOrder.getOrderId(),1,licenceOrder.getEnterpriseId(),licenceOrder.getWxEnterpriseId(),"系统",1);
        //保存微信订单id
        licenceOrderService.saveTransactionCode(licenceOrder.getOrderId(), null,null,transactionId,payTime);
        //生成企业微信订单
//        payLicenceOrder(licenceOrder.getOrderId(),licenceOrder.getWxEnterpriseId());
        //查询gic品牌名称
        String enterpriseName="";
        String wxEnterpriseName="";
        EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(licenceOrder.getEnterpriseId());
        if (ObjectUtil.isNotNull(enterpriseDTO)){
            enterpriseName=enterpriseDTO.getEnterpriseName();
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(licenceOrder.getWxEnterpriseId());
        if (ObjectUtil.isNotNull(wxEnterpriseDTO)){
            wxEnterpriseName=wxEnterpriseDTO.getCorpName();
        }
        String dateToStr = DateUtil.dateToStr(new Date(), DateUtil.FORMAT_DATETIME_19);
        String msg = "标题：企微账号许可通知\n品牌名称:"+enterpriseName+"\n"+"企业名称："+wxEnterpriseName+"\n"+"消息内容："+dateToStr+"订单编号为"+licenceOrder.getTransactionId()+",请尽快处理";
        DingUtils.send(msg,"https://oapi.dingtalk.com/robot/send?access_token=63074c9fc1ac2c00aa62477facdf16e5fa2388cf6a621edba1fb05d6e55d031f",false);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> qywxCallBack(String params) {
        logger.info("接收企业微信回调参数：{}",JSON.toJSONString(params));
        if(StrUtil.isBlank(params)){
            return ServiceResponse.success(false);
        }
        JSONObject object = JSONObject.parseObject(params, JSONObject.class);
        String infoType = object.getString("infoType");
        String openCorpId = object.getString("openCorpId");
        String orderId = object.getString("orderId");
        String timeStamp = object.getString("timeStamp");
        Integer orderStatus = object.getInteger("orderStatus");
        String buyerUserId = object.getString("buyerUserId");
        TabHaobanLicenceOrder order = licenceOrderService.selectByQywxOrderId(orderId, openCorpId);
        if ("license_pay_success".equals(infoType)){
            //支付回调
            //更新企业微信订单状态
            licenceOrderService.updateLicenceOrderType(order.getOrderId(), 1,order.getEnterpriseId(),order.getWxEnterpriseId(),"系统",0);
        }
        if ("license_refund".equals(infoType)){
            //退款回调 todo 根据退款状态判断
            licenceOrderService.updateLicenceOrderType(order.getOrderId(), 6,order.getEnterpriseId(),order.getWxEnterpriseId(),"系统",0);
        }
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> saveTransactionCode(Long orderId, String transactionCode, String prepayId, String wxOrderId,Date payTime) {
        licenceOrderService.saveTransactionCode(orderId,transactionCode,prepayId,wxOrderId,payTime);
        return ServiceResponse.success(true);
    }
}
